@@ -17,12 +17,14 @@
 
 use crate::config as cfg;
 use crate::layout;
+use crate::shared::ipc;
+use crate::shared::ipc::NodeMethods;
 use crate::tree as t;
-use crate::tree::NodeMethods;
 use crate::util;
 use crate::util::DisplayFormat;
-use lazy_static::lazy_static;
+use once_cell::sync::Lazy;
 use rand::prelude::SliceRandom;
+use regex::Regex;
 use serde::{Deserialize, Serialize};
 use std::collections::HashMap;
 use std::sync::atomic;
@@ -482,19 +484,6 @@ fn quit_window_by_id(id: i64) {
     run_sway_command(&[format!("[con_id={}]", id).as_str(), "kill"]);
 }
 
-pub fn get_tree(include_scratchpad: bool) -> s::Node {
-    match s::Connection::new() {
-        Ok(mut con) => {
-            let mut root = con.get_tree().expect("Got no root node");
-            if !include_scratchpad {
-                root.nodes.retain(|o| !o.is_scratchpad());
-            }
-            root
-        }
-        Err(err) => panic!("{}", err),
-    }
-}
-
 pub fn get_outputs() -> Vec<s::Output> {
     match s::Connection::new() {
         Ok(mut con) => con.get_outputs().expect("Got no outputs"),
@@ -512,7 +501,7 @@ pub fn switch_to_app_or_urgent_or_lru_window(
     name: Option<&str>,
     extra_props: &HashMap<i64, t::ExtraProps>,
 ) {
-    let root = get_tree(false);
+    let root = ipc::get_root_node(false);
     let tree = t::get_tree(&root, extra_props);
     let wins = tree.get_windows();
     let app_win =
@@ -524,7 +513,7 @@ pub fn switch_to_mark_or_urgent_or_lru_window(
     con_mark: Option<&str>,
     extra_props: &HashMap<i64, t::ExtraProps>,
 ) {
-    let root = get_tree(false);
+    let root = ipc::get_root_node(false);
     let tree = t::get_tree(&root, extra_props);
     let wins = tree.get_windows();
     let marked_win = con_mark.and_then(|mark| {
@@ -550,10 +539,8 @@ pub fn focus_win_if_not_focused(
     }
 }
 
-lazy_static! {
-    static ref DIGIT_AND_NAME: regex::Regex =
-        regex::Regex::new(r"^(\d):(.*)").unwrap();
-}
+static DIGIT_AND_NAME: Lazy<Regex> =
+    Lazy::new(|| Regex::new(r"^(\d):(.*)").unwrap());
 
 fn create_workspace(ws_name: &str) {
     if DIGIT_AND_NAME.is_match(ws_name) {
@@ -563,12 +550,10 @@ fn create_workspace(ws_name: &str) {
     }
 }
 
-lazy_static! {
-    static ref SPECIAL_WORKSPACE: regex::Regex =
-        regex::Regex::new(r"^#*w:(.*)").unwrap();
-    static ref SPECIAL_SWAY: regex::Regex =
-        regex::Regex::new(r"^#*s:(.*)").unwrap();
-}
+static SPECIAL_WORKSPACE: Lazy<Regex> =
+    Lazy::new(|| Regex::new(r"^#*w:(.*)").unwrap());
+static SPECIAL_SWAY: Lazy<regex::Regex> =
+    Lazy::new(|| Regex::new(r"^#*s:(.*)").unwrap());
 
 fn chop_workspace_shortcut(input: &str) -> &str {
     match SPECIAL_WORKSPACE.captures(input) {
@@ -600,17 +585,17 @@ fn handle_non_matching_input(input: &str) {
 fn select_and_focus(prompt: &str, choices: &[t::DisplayNode]) {
     match util::select_from_menu(prompt, choices) {
         Ok(tn) => match tn.node.get_type() {
-            t::Type::Output => {
+            ipc::Type::Output => {
                 if !tn.node.is_scratchpad() {
                     run_sway_command(&["focus output", tn.node.get_name()]);
                 }
             }
-            t::Type::Workspace => {
+            ipc::Type::Workspace => {
                 if !tn.node.is_scratchpad() {
                     run_sway_command(&["workspace", tn.node.get_name()]);
                 }
             }
-            t::Type::Window | t::Type::Container => {
+            ipc::Type::Window | ipc::Type::Container => {
                 focus_window_by_id(tn.node.id)
             }
             t => {
@@ -624,25 +609,25 @@ fn select_and_focus(prompt: &str, choices: &[t::DisplayNode]) {
 }
 
 pub fn switch_window(extra_props: &HashMap<i64, t::ExtraProps>) {
-    let root = get_tree(true);
+    let root = ipc::get_root_node(true);
     let tree = t::get_tree(&root, extra_props);
     select_and_focus("Select window", &tree.get_windows());
 }
 
 pub fn switch_workspace(extra_props: &HashMap<i64, t::ExtraProps>) {
-    let root = get_tree(false);
+    let root = ipc::get_root_node(false);
     let tree = t::get_tree(&root, extra_props);
     select_and_focus("Select workspace", &tree.get_workspaces());
 }
 
 pub fn switch_output(extra_props: &HashMap<i64, t::ExtraProps>) {
-    let root = get_tree(false);
+    let root = ipc::get_root_node(false);
     let tree = t::get_tree(&root, extra_props);
     select_and_focus("Select output", &tree.get_outputs());
 }
 
 pub fn switch_workspace_or_window(extra_props: &HashMap<i64, t::ExtraProps>) {
-    let root = get_tree(true);
+    let root = ipc::get_root_node(true);
     let tree = t::get_tree(&root, extra_props);
     select_and_focus(
         "Select workspace or window",
@@ -653,7 +638,7 @@ pub fn switch_workspace_or_window(extra_props: &HashMap<i64, t::ExtraProps>) {
 pub fn switch_workspace_container_or_window(
     extra_props: &HashMap<i64, t::ExtraProps>,
 ) {
-    let root = get_tree(true);
+    let root = ipc::get_root_node(true);
     let tree = t::get_tree(&root, extra_props);
     select_and_focus(
         "Select workspace, container or window",
@@ -662,7 +647,7 @@ pub fn switch_workspace_container_or_window(
 }
 
 pub fn switch_to(extra_props: &HashMap<i64, t::ExtraProps>) {
-    let root = get_tree(true);
+    let root = ipc::get_root_node(true);
     let tree = t::get_tree(&root, extra_props);
     select_and_focus(
         "Select output, workspace, container or window",
@@ -687,14 +672,14 @@ fn kill_process_by_pid(pid: Option<i32>) {
 fn select_and_quit(prompt: &str, choices: &[t::DisplayNode], kill: bool) {
     if let Ok(tn) = util::select_from_menu(prompt, choices) {
         match tn.node.get_type() {
-            t::Type::Workspace | t::Type::Container => {
+            ipc::Type::Workspace | ipc::Type::Container => {
                 for win in
-                    tn.node.iter().filter(|n| n.get_type() == t::Type::Window)
+                    tn.node.iter().filter(|n| n.get_type() == ipc::Type::Window)
                 {
                     quit_window_by_id(win.id)
                 }
             }
-            t::Type::Window => {
+            ipc::Type::Window => {
                 if kill {
                     kill_process_by_pid(tn.node.pid)
                 } else {
@@ -709,13 +694,13 @@ fn select_and_quit(prompt: &str, choices: &[t::DisplayNode], kill: bool) {
 }
 
 pub fn quit_window(extra_props: &HashMap<i64, t::ExtraProps>, kill: bool) {
-    let root = get_tree(true);
+    let root = ipc::get_root_node(true);
     let tree = t::get_tree(&root, extra_props);
     select_and_quit("Quit window", &tree.get_windows(), kill);
 }
 
 pub fn quit_workspace_or_window(extra_props: &HashMap<i64, t::ExtraProps>) {
-    let root = get_tree(true);
+    let root = ipc::get_root_node(true);
     let tree = t::get_tree(&root, extra_props);
     select_and_quit(
         "Quit workspace or window",
@@ -727,7 +712,7 @@ pub fn quit_workspace_or_window(extra_props: &HashMap<i64, t::ExtraProps>) {
 pub fn quit_workspace_container_or_window(
     extra_props: &HashMap<i64, t::ExtraProps>,
 ) {
-    let root = get_tree(true);
+    let root = ipc::get_root_node(true);
     let tree = t::get_tree(&root, extra_props);
     select_and_quit(
         "Quit workspace, container or window",
@@ -765,7 +750,7 @@ fn move_focused_to_container_or_window(id: i64) {
 fn select_and_move_focused_to(prompt: &str, choices: &[t::DisplayNode]) {
     match util::select_from_menu(prompt, choices) {
         Ok(tn) => match tn.node.get_type() {
-            t::Type::Output => {
+            ipc::Type::Output => {
                 if tn.node.is_scratchpad() {
                     run_sway_command_1("move container to scratchpad")
                 } else {
@@ -775,14 +760,14 @@ fn select_and_move_focused_to(prompt: &str, choices: &[t::DisplayNode]) {
                     ])
                 }
             }
-            t::Type::Workspace => {
+            ipc::Type::Workspace => {
                 if tn.node.is_scratchpad() {
                     run_sway_command_1("move container to scratchpad")
                 } else {
                     move_focused_to_workspace_1(tn.node.get_name())
                 }
             }
-            t::Type::Container | t::Type::Window => {
+            ipc::Type::Container | ipc::Type::Window => {
                 move_focused_to_container_or_window(tn.node.id)
             }
             t => log::error!("Cannot move focused to {:?}", t),
@@ -795,7 +780,7 @@ fn select_and_move_focused_to(prompt: &str, choices: &[t::DisplayNode]) {
 }
 
 pub fn move_focused_to_workspace(extra_props: &HashMap<i64, t::ExtraProps>) {
-    let root = get_tree(true);
+    let root = ipc::get_root_node(true);
     let tree = t::get_tree(&root, extra_props);
     select_and_move_focused_to(
         "Move focused container to workspace",
@@ -804,7 +789,7 @@ pub fn move_focused_to_workspace(extra_props: &HashMap<i64, t::ExtraProps>) {
 }
 
 pub fn move_focused_to(extra_props: &HashMap<i64, t::ExtraProps>) {
-    let root = get_tree(true);
+    let root = ipc::get_root_node(true);
     let tree = t::get_tree(&root, extra_props);
     select_and_move_focused_to(
         "Move focused container to workspace or container",
@@ -813,14 +798,14 @@ pub fn move_focused_to(extra_props: &HashMap<i64, t::ExtraProps>) {
 }
 
 pub fn swap_focused_with(extra_props: &HashMap<i64, t::ExtraProps>) {
-    let root = get_tree(true);
+    let root = ipc::get_root_node(true);
     let tree = t::get_tree(&root, extra_props);
     match util::select_from_menu(
         "Swap focused with",
         &tree.get_workspaces_containers_and_windows(),
     ) {
         Ok(tn) => match tn.node.get_type() {
-            t::Type::Workspace | t::Type::Container | t::Type::Window => {
+            ipc::Type::Workspace | ipc::Type::Container | ipc::Type::Window => {
                 run_sway_command(&[
                     "swap",
                     "container",
@@ -849,14 +834,14 @@ pub fn focus_window_in_direction(
     extra_props: &HashMap<i64, t::ExtraProps>,
     pred: Box<dyn Fn(&t::DisplayNode) -> bool>,
 ) {
-    let root = get_tree(false);
+    let root = ipc::get_root_node(false);
     let tree = t::get_tree(&root, extra_props);
     let mut wins = tree.get_windows();
 
     if consider_wins == &ConsiderWindows::CurrentWorkspace {
         let cur_ws = tree.get_current_workspace();
         wins.retain(|w| {
-            tree.get_parent_node_of_type(w.node.id, t::Type::Workspace)
+            tree.get_parent_node_of_type(w.node.id, ipc::Type::Workspace)
                 .unwrap()
                 .id
                 == cur_ws.id
@@ -903,7 +888,7 @@ pub fn focus_window_of_same_layout_in_direction(
     consider_wins: &ConsiderWindows,
     extra_props: &HashMap<i64, t::ExtraProps>,
 ) {
-    let root = get_tree(false);
+    let root = ipc::get_root_node(false);
     let tree = t::get_tree(&root, extra_props);
     let wins = tree.get_windows();
     let cur_win = wins.iter().find(|w| w.node.focused);
@@ -1015,8 +1000,8 @@ fn tab_current_workspace(floating: &ConsiderFloating) {
 }
 
 fn toggle_tab_tile_current_workspace(floating: &ConsiderFloating) {
-    let tree = get_tree(false);
-    let workspaces = tree.nodes_of_type(t::Type::Workspace);
+    let tree = ipc::get_root_node(false);
+    let workspaces = tree.nodes_of_type(ipc::Type::Workspace);
     let cur_ws = workspaces.iter().find(|w| w.is_current()).unwrap();
     if cur_ws.layout == s::NodeLayout::Tabbed {
         tile_current_workspace(floating, true);
