@@ -15,13 +15,9 @@
 
 //! TOML configuration for swayr.
 
-use directories::ProjectDirs;
+use crate::shared::cfg;
 use serde::{Deserialize, Serialize};
 use std::collections::HashMap;
-use std::fs::DirBuilder;
-use std::fs::OpenOptions;
-use std::io::{Read, Write};
-use std::path::Path;
 
 #[derive(Debug, Serialize, Deserialize)]
 pub struct Config {
@@ -304,72 +300,12 @@ impl Default for Config {
     }
 }
 
-fn get_config_file_path() -> Box<Path> {
-    let proj_dirs = ProjectDirs::from("", "", "swayr").expect("");
-    let user_config_dir = proj_dirs.config_dir();
-    if !user_config_dir.exists() {
-        let sys_config_file = Path::new("/etc/xdg/swayr/config.toml");
-        if sys_config_file.exists() {
-            return sys_config_file.into();
-        }
-        DirBuilder::new()
-            .recursive(true)
-            .create(user_config_dir)
-            .unwrap();
-    }
-    user_config_dir.join("config.toml").into_boxed_path()
-}
-
-pub fn save_config(cfg: Config) {
-    let path = get_config_file_path();
-    let content =
-        toml::to_string_pretty(&cfg).expect("Cannot serialize config.");
-    let mut file = OpenOptions::new()
-        .read(false)
-        .write(true)
-        .create(true)
-        .open(path)
-        .unwrap();
-    file.write_all(content.as_str().as_bytes()).unwrap();
-}
-
 pub fn load_config() -> Config {
-    let path = get_config_file_path();
-    if !path.exists() {
-        save_config(Config::default());
-        // Tell the user that a fresh default config has been created.
-        std::process::Command::new("swaynag")
-            .arg("--background")
-            .arg("00FF44")
-            .arg("--text")
-            .arg("0000CC")
-            .arg("--message")
-            .arg(
-                "Welcome to swayr! ".to_owned()
-                    + "I've created a fresh config for use with wofi for you in "
-                    + &path.to_string_lossy()
-                    + ". Adapt it to your needs.",
-            )
-            .arg("--type")
-            .arg("warning")
-            .arg("--dismiss-button")
-            .arg("Thanks!")
-            .spawn()
-            .ok();
-    }
-    let mut file = OpenOptions::new()
-        .read(true)
-        .write(false)
-        .create(false)
-        .open(path)
-        .unwrap();
-    let mut buf: String = String::new();
-    file.read_to_string(&mut buf).unwrap();
-    toml::from_str(&buf).expect("Invalid config.")
+    cfg::load_config::<Config>("swayr")
 }
 
 #[test]
-fn test_load_config() {
-    let cfg = load_config();
+fn test_load_swayr_config() {
+    let cfg = cfg::load_config::<Config>("swayr");
     println!("{:?}", cfg);
 }
