@@ -13,11 +13,6 @@
 // You should have received a copy of the GNU General Public License along with
 // this program.  If not, see <https://www.gnu.org/licenses/>.
 
-// TODO: Possibly just include README.md when this feature is in the release
-// channel.
-//
-// #![doc(include = "../README.md")]
-
 //! **Swayr** is a LRU window-switcher and more for the sway window manager.
 //! It consists of a demon, and a client.  The demon `swayrd` records
 //! window/workspace creations, deletions, and focus changes using sway's JSON
@@ -28,6 +23,6 @@ pub mod cmds;
 pub mod config;
 pub mod demon;
 pub mod layout;
-pub mod rtfmt;
+pub mod shared;
 pub mod tree;
 pub mod util;
