@@ -16,10 +16,13 @@
 //! Convenience data structures built from the IPC structs.
 
 use crate::config;
-use crate::rtfmt;
+use crate::shared::fmt::subst_placeholders;
+use crate::shared::ipc;
+use crate::shared::ipc::NodeMethods;
 use crate::util;
 use crate::util::DisplayFormat;
-use lazy_static::lazy_static;
+use once_cell::sync::Lazy;
+use regex::Regex;
 use serde::{Deserialize, Serialize};
 use std::cell::RefCell;
 use std::cmp;
@@ -27,141 +30,6 @@ use std::collections::HashMap;
 use std::rc::Rc;
 use swayipc as s;
 
-/// Immutable Node Iterator
-///
-/// Iterates nodes in depth-first order, tiled nodes before floating nodes.
-pub struct NodeIter<'a> {
-    stack: Vec<&'a s::Node>,
-}
-
-impl<'a> NodeIter<'a> {
-    fn new(node: &'a s::Node) -> NodeIter {
-        NodeIter { stack: vec![node] }
-    }
-}
-
-impl<'a> Iterator for NodeIter<'a> {
-    type Item = &'a s::Node;
-
-    fn next(&mut self) -> Option<Self::Item> {
-        if let Some(node) = self.stack.pop() {
-            for n in &node.floating_nodes {
-                self.stack.push(n);
-            }
-            for n in &node.nodes {
-                self.stack.push(n);
-            }
-            Some(node)
-        } else {
-            None
-        }
-    }
-}
-
-#[derive(Debug, PartialEq, Eq, Clone)]
-pub enum Type {
-    Root,
-    Output,
-    Workspace,
-    Container,
-    Window,
-}
-
-/// Extension methods for [`swayipc::Node`].
-pub trait NodeMethods {
-    fn iter(&self) -> NodeIter;
-    fn get_type(&self) -> Type;
-    fn get_app_name(&self) -> &str;
-    fn nodes_of_type(&self, t: Type) -> Vec<&s::Node>;
-    fn get_name(&self) -> &str;
-    fn is_scratchpad(&self) -> bool;
-    fn is_floating(&self) -> bool;
-    fn is_current(&self) -> bool;
-}
-
-impl NodeMethods for s::Node {
-    fn iter(&self) -> NodeIter {
-        NodeIter::new(self)
-    }
-
-    fn get_type(&self) -> Type {
-        match self.node_type {
-            s::NodeType::Root => Type::Root,
-            s::NodeType::Output => Type::Output,
-            s::NodeType::Workspace => Type::Workspace,
-            s::NodeType::FloatingCon => Type::Window,
-            _ => {
-                if self.node_type == s::NodeType::Con
-                    && self.name.is_none()
-                    && self.app_id.is_none()
-                    && self.pid.is_none()
-                    && self.shell.is_none()
-                    && self.window_properties.is_none()
-                    && self.layout != s::NodeLayout::None
-                {
-                    Type::Container
-                } else if (self.node_type == s::NodeType::Con
-                    || self.node_type == s::NodeType::FloatingCon)
-                    // Apparently there can be windows without app_id, name,
-                    // and window_properties.class, e.g., dolphin-emu-nogui.
-                    && self.pid.is_some()
-                // FIXME: While technically correct, old sway versions (up to
-                // at least sway-1.4) don't expose shell in IPC.  So comment in
-                // again when all major distros have a recent enough sway
-                // package.
-                //&& self.shell.is_some()
-                {
-                    Type::Window
-                } else {
-                    panic!(
-                        "Don't know type of node with id {} and node_type {:?}\n{:?}",
-                        self.id, self.node_type, self
-                    )
-                }
-            }
-        }
-    }
-
-    fn get_name(&self) -> &str {
-        if let Some(name) = &self.name {
-            name.as_ref()
-        } else {
-            "<unnamed>"
-        }
-    }
-
-    fn get_app_name(&self) -> &str {
-        if let Some(app_id) = &self.app_id {
-            app_id
-        } else if let Some(wp_class) = self
-            .window_properties
-            .as_ref()
-            .and_then(|wp| wp.class.as_ref())
-        {
-            wp_class
-        } else {
-            "<unknown_app>"
-        }
-    }
-
-    fn is_scratchpad(&self) -> bool {
-        let name = self.get_name();
-        name.eq("__i3") || name.eq("__i3_scratch")
-    }
-
-    fn nodes_of_type(&self, t: Type) -> Vec<&s::Node> {
-        self.iter().filter(|n| n.get_type() == t).collect()
-    }
-
-    fn is_floating(&self) -> bool {
-        self.node_type == s::NodeType::FloatingCon
-    }
-
-    fn is_current(&self) -> bool {
-        self.iter().any(|n| n.focused)
-    }
-}
-
 /// Extra properties gathered by swayrd for windows and workspaces.
 #[derive(Copy, Clone, Debug, Deserialize, Serialize)]
 pub struct ExtraProps {
@@ -203,7 +71,7 @@ impl<'a> Tree<'a> {
     pub fn get_parent_node_of_type(
         &self,
         id: i64,
-        t: Type,
+        t: ipc::Type,
     ) -> Option<&&s::Node> {
         let n = self.get_node_by_id(id);
         if n.get_type() == t {
@@ -228,14 +96,14 @@ impl<'a> Tree<'a> {
     fn sorted_nodes_of_type_1(
         &self,
         node: &'a s::Node,
-        t: Type,
+        t: ipc::Type,
     ) -> Vec<&s::Node> {
         let mut v: Vec<&s::Node> = node.nodes_of_type(t);
         self.sort_by_urgency_and_lru_time_1(&mut v);
         v
     }
 
-    fn sorted_nodes_of_type(&self, t: Type) -> Vec<&s::Node> {
+    fn sorted_nodes_of_type(&self, t: ipc::Type) -> Vec<&s::Node> {
         self.sorted_nodes_of_type_1(self.root, t)
     }
 
@@ -256,7 +124,7 @@ impl<'a> Tree<'a> {
     pub fn get_current_workspace(&self) -> &s::Node {
         self.root
             .iter()
-            .find(|n| n.get_type() == Type::Workspace && n.is_current())
+            .find(|n| n.get_type() == ipc::Type::Workspace && n.is_current())
             .expect("No current Workspace")
     }
 
@@ -264,13 +132,13 @@ impl<'a> Tree<'a> {
         let outputs: Vec<&s::Node> = self
             .root
             .iter()
-            .filter(|n| n.get_type() == Type::Output && !n.is_scratchpad())
+            .filter(|n| n.get_type() == ipc::Type::Output && !n.is_scratchpad())
             .collect();
         self.as_display_nodes(&outputs, IndentLevel::Fixed(0))
     }
 
     pub fn get_workspaces(&self) -> Vec<DisplayNode> {
-        let mut v = self.sorted_nodes_of_type(Type::Workspace);
+        let mut v = self.sorted_nodes_of_type(ipc::Type::Workspace);
         if !v.is_empty() {
             v.rotate_left(1);
         }
@@ -278,7 +146,7 @@ impl<'a> Tree<'a> {
     }
 
     pub fn get_windows(&self) -> Vec<DisplayNode> {
-        let mut v = self.sorted_nodes_of_type(Type::Window);
+        let mut v = self.sorted_nodes_of_type(ipc::Type::Window);
         // Rotate, but only non-urgent windows.  Those should stay at the front
         // as they are the most likely switch candidates.
         let mut x;
@@ -302,12 +170,12 @@ impl<'a> Tree<'a> {
     }
 
     pub fn get_workspaces_and_windows(&self) -> Vec<DisplayNode> {
-        let workspaces = self.sorted_nodes_of_type(Type::Workspace);
+        let workspaces = self.sorted_nodes_of_type(ipc::Type::Workspace);
         let mut first = true;
         let mut v = vec![];
         for ws in workspaces {
             v.push(ws);
-            let mut wins = self.sorted_nodes_of_type_1(ws, Type::Window);
+            let mut wins = self.sorted_nodes_of_type_1(ws, ipc::Type::Window);
             if first && !wins.is_empty() {
                 wins.rotate_left(1);
                 first = false;
@@ -318,7 +186,7 @@ impl<'a> Tree<'a> {
         self.as_display_nodes(&v, IndentLevel::WorkspacesZeroWindowsOne)
     }
 
-    fn sort_by_urgency_and_lru_time_1(&self, v: &mut Vec<&s::Node>) {
+    fn sort_by_urgency_and_lru_time_1(&self, v: &mut [&s::Node]) {
         v.sort_by(|a, b| {
             if a.urgent && !b.urgent {
                 cmp::Ordering::Less
@@ -351,7 +219,7 @@ impl<'a> Tree<'a> {
     pub fn get_outputs_workspaces_containers_and_windows(
         &self,
     ) -> Vec<DisplayNode> {
-        let outputs = self.sorted_nodes_of_type(Type::Output);
+        let outputs = self.sorted_nodes_of_type(ipc::Type::Output);
         let v: Rc<RefCell<Vec<&s::Node>>> = Rc::new(RefCell::new(vec![]));
         for o in outputs {
             self.push_subtree_sorted(o, Rc::clone(&v));
@@ -362,7 +230,7 @@ impl<'a> Tree<'a> {
     }
 
     pub fn get_workspaces_containers_and_windows(&self) -> Vec<DisplayNode> {
-        let workspaces = self.sorted_nodes_of_type(Type::Workspace);
+        let workspaces = self.sorted_nodes_of_type(ipc::Type::Workspace);
         let v: Rc<RefCell<Vec<&s::Node>>> = Rc::new(RefCell::new(vec![]));
         for ws in workspaces {
             self.push_subtree_sorted(ws, Rc::clone(&v));
@@ -429,24 +297,8 @@ pub fn get_tree<'a>(
     }
 }
 
-lazy_static! {
-    static ref APP_NAME_AND_VERSION_RX: regex::Regex =
-        regex::Regex::new("(.+)(-[0-9.]+)").unwrap();
-    static ref PLACEHOLDER_RX: regex::Regex = regex::Regex::new(
-        r"\{(?P<name>[^}:]+)(?::(?P<fmtstr>\{[^}]*\})(?P<clipstr>[^}]*))?\}"
-    )
-    .unwrap();
-}
-
-fn maybe_html_escape(do_it: bool, text: String) -> String {
-    if do_it {
-        text.replace('<', "&lt;")
-            .replace('>', "&gt;")
-            .replace('&', "&amp;")
-    } else {
-        text
-    }
-}
+static APP_NAME_AND_VERSION_RX: Lazy<Regex> =
+    Lazy::new(|| Regex::new("(.+)(-[0-9.]+)").unwrap());
 
 fn format_marks(marks: &[String]) -> String {
     if marks.is_empty() {
@@ -473,14 +325,17 @@ impl DisplayFormat for DisplayNode<'_> {
             APP_NAME_AND_VERSION_RX.replace(self.node.get_app_name(), "$1");
 
         let fmt = match self.node.get_type() {
-            Type::Root => String::from("Cannot format Root"),
-            Type::Output => cfg.get_format_output_format(),
-            Type::Workspace => cfg.get_format_workspace_format(),
-            Type::Container => cfg.get_format_container_format(),
-            Type::Window => cfg.get_format_window_format(),
+            ipc::Type::Root => String::from("Cannot format Root"),
+            ipc::Type::Output => cfg.get_format_output_format(),
+            ipc::Type::Workspace => cfg.get_format_workspace_format(),
+            ipc::Type::Container => cfg.get_format_container_format(),
+            ipc::Type::Window => cfg.get_format_window_format(),
         };
         let fmt = fmt
-            .replace("{indent}", &indent.repeat(self.get_indent_level()))
+            .replace(
+                "{indent}",
+                indent.repeat(self.get_indent_level()).as_str(),
+            )
             .replace(
                 "{urgency_start}",
                 if self.node.urgent {
@@ -515,36 +370,21 @@ impl DisplayFormat for DisplayNode<'_> {
                     .as_str(),
             );
 
-        PLACEHOLDER_RX
-            .replace_all(&fmt, |caps: &regex::Captures| {
-                let value = match &caps["name"] {
-                    "id" => self.node.id.to_string(),
-                    "app_name" => self.node.get_app_name().to_string(),
-                    "layout" => format!("{:?}", self.node.layout),
-                    "name" | "title" => self.node.get_name().to_string(),
-                    "output_name" => self
-                        .tree
-                        .get_parent_node_of_type(self.node.id, Type::Output)
-                        .map_or("<no_output>", |w| w.get_name())
-                        .to_string(),
-                    "workspace_name" => self
-                        .tree
-                        .get_parent_node_of_type(self.node.id, Type::Workspace)
-                        .map_or("<no_workspace>", |w| w.get_name())
-                        .to_string(),
-                    "marks" => format_marks(&self.node.marks),
-                    _ => caps[0].to_string(),
-                };
-                let fmt_str = caps.name("fmtstr").map_or("{}", |m| m.as_str());
-                let clipped_str =
-                    caps.name("clipstr").map_or("", |m| m.as_str());
-
-                maybe_html_escape(
-                    html_escape,
-                    rtfmt::format(fmt_str, &value, clipped_str),
-                )
-            })
-            .into()
+        subst_placeholders!(&fmt, html_escape, {
+            "id" => self.node.id,
+            "app_name" => self.node.get_app_name(),
+            "layout" => format!("{:?}", self.node.layout),
+            "name" | "title" => self.node.get_name(),
+            "output_name" => self
+                .tree
+                .get_parent_node_of_type(self.node.id, ipc::Type::Output)
+                .map_or("<no_output>", |w| w.get_name()),
+            "workspace_name" => self
+                .tree
+                .get_parent_node_of_type(self.node.id, ipc::Type::Workspace)
+                .map_or("<no_workspace>", |w| w.get_name()),
+            "marks" => format_marks(&self.node.marks),
+        })
     }
 
     fn get_indent_level(&self) -> usize {
@@ -552,8 +392,8 @@ impl DisplayFormat for DisplayNode<'_> {
             IndentLevel::Fixed(level) => level as usize,
             IndentLevel::WorkspacesZeroWindowsOne => {
                 match self.node.get_type(){
-                    Type::Workspace => 0,
-                    Type::Window => 1,
+                    ipc::Type::Workspace => 0,
+                    ipc::Type::Window => 1,
                     _ => panic!("Only Workspaces and Windows expected. File a bug report!")
                 }
             }
@@ -573,26 +413,3 @@ impl DisplayFormat for DisplayNode<'_> {
         }
     }
 }
-
-#[test]
-fn test_placeholder_rx() {
-    let caps = PLACEHOLDER_RX.captures("Hello, {place}!").unwrap();
-    assert_eq!(caps.name("name").unwrap().as_str(), "place");
-    assert_eq!(caps.name("fmtstr"), None);
-    assert_eq!(caps.name("clipstr"), None);
-
-    let caps = PLACEHOLDER_RX.captures("Hi, {place:{:>10.10}}!").unwrap();
-    assert_eq!(caps.name("name").unwrap().as_str(), "place");
-    assert_eq!(caps.name("fmtstr").unwrap().as_str(), "{:>10.10}");
-    assert_eq!(caps.name("clipstr").unwrap().as_str(), "");
-
-    let caps = PLACEHOLDER_RX.captures("Hello, {place:{:.5}…}!").unwrap();
-    assert_eq!(caps.name("name").unwrap().as_str(), "place");
-    assert_eq!(caps.name("fmtstr").unwrap().as_str(), "{:.5}");
-    assert_eq!(caps.name("clipstr").unwrap().as_str(), "…");
-
-    let caps = PLACEHOLDER_RX.captures("Hello, {place:{:.5}...}!").unwrap();
-    assert_eq!(caps.name("name").unwrap().as_str(), "place");
-    assert_eq!(caps.name("fmtstr").unwrap().as_str(), "{:.5}");
-    assert_eq!(caps.name("clipstr").unwrap().as_str(), "...");
-}
